<?php
namespace EMS\PortalFuncionario\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\{ Request, Response };
use Zeedhi\Framework\DTO\Response\{ Error, Message };
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Geral\Services\{MarcacaoService, OcorrenciaService};

class MarcacaoController extends Crud {

    /** @var MarcacaoService */
    protected $marcacaoService;

    public function __construct(EntityManager $entityManager, MarcacaoService $marcacaoService, OcorrenciaService $ocorrenciaService) {
      $this->entityManager      = $entityManager;
      $this->marcacaoService    = $marcacaoService;
      $this->ocorrenciaService  = $ocorrenciaService;
    }

    public function registraMarcacaoIntervalo(
        Request\DataSet $request,
        Response $response
    ) {
        try {
            $rows = $request->getDataSet()->getRows();
            $row = current($rows)->getArrayCopy();

            $success = $this->marcacaoService->registraMarcacaoIntervalo($row);
            $response->addDataSet(new DataSet('setMarcacaoIntervalo', ['success' => $success]));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function registraMarcacaoAlmoco(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $row = current($rows)->getArrayCopy();

            $success = $this->marcacaoService->registraMarcacaoAlmoco($row);
            $response->addDataSet(new DataSet('setMarcacaoAlmoco', ['success' => $success]));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function registraOcorrenciaPonto(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow()->getArrayCopy();

            $success = $this->marcacaoService->registraOcorrenciaPonto($row);

            $response->addDataSet(new DataSet('setOcorrenciaPonto', ['success' => $success]));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function validaMarcacaoDistancia(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $row = current($rows);
    
            $validacoes = $this->marcacaoService->getValidacaoMarcacaoDistancia(
                $row['NRVINCULOM'],
                $row['NRORG'],
                $row['DSLATITUDE'],
                $row['DSLONGITUDE']
            );
            $response->addDataSet(new DataSet('validacoesMarcacao', array('validacoesMarcacao' => $validacoes)));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getParesMarcacoesDia(
        DTO\Request\Filter  $request,
        DTO\Response        $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg     = (int) $row['NRORG'];
            $nrVinculo = (int) $row['NRVINCULOM'];
            $dia       = $row['DIA'];

            $dataSet = $this->marcacaoService->
                getParesMarcacoesDia($nrOrg, $nrVinculo, $dia);

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function saveParesMarcacao(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg             = (int)      $row['NRORG'];
            $nrVinculo         = (int)      $row['NRVINCULOM'];
            $dia               = (array)    $row['DIA'];
            $hrEntrada         = (string)   $row['HRENTRADA'];
            $hrSaida           = (string)   $row['HRSAIDA'];
            $idDiaInicio       = (string)   $row['DIA_INICIO'];
            $idDiaFim          = (string)   $row['DIA_FIM'];

            $nrMarcacoes = $this->marcacaoService->saveParesMarcacao(
                $nrOrg,
                $nrVinculo,
                $dia,
                $hrEntrada,
                $hrSaida,
                $idDiaInicio,
                $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeParesMarcacao(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg             = (int)      $row['NRORG'];
            $nrVinculo         = (int)      $row['NRVINCULOM'];
            $dia               = (array)    $row['DIA'];
            $hrEntrada         = (string)   $row['HRENTRADA'];
            $hrSaida           = (string)   $row['HRSAIDA'];

            $nrMarcacoes = $this->marcacaoService->removeParesMarcacao(
                $nrOrg,
                $nrVinculo,
                $dia,
                $hrEntrada,
                $hrSaida
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function removeMarcacao(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg              = (int)      $row['NRORG'];
            $nrVinculo          = (int)      $row['NRVINCULOM'];
            $nrMarcacaoponto    = (string)   $row['NRMARCACAOPONTO'];

            $nrMarcacoes = $this->marcacaoService->removeMarcacao(
                $nrOrg,
                $nrVinculo,
                $nrMarcacaoponto
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function editParesMarcacao(
        DTO\Request\Row     $request,
        DTO\Response        $response
    ) {

        try {
            $row = $request->getRow();

            $nrOrg          = (int)     $row['NRORG'];
            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $hrentrada      = (string)  $row['HRENTRADA'];
            $hrsaida        = (string)  $row['HRSAIDA'];
            $dates          = (array)   $row['DATE'];
            $novahrentrada  = (string)  $row['NOVAHRENTRADA'];
            $novahrsaida    = (string)  $row['NOVAHRSAIDA'];

            $nrMarcacoes = $this->marcacaoService->editParesMarcacao(
                $nrOrg,
                $nrVinculo,
                $hrentrada,
                $hrsaida,
                $dates,
                $novahrentrada,
                $novahrsaida
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getServerTime(
        DTO\Request\Filter $request,
        DTO\Response $response
    ) {
        try {
            $serverDateTime = $this->marcacaoService->getServerTime();

            $response->addDataSet(new DataSet('data', $serverDateTime));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function fixaEntradaSaida(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg             = (int)    $row['NRORG'];
            $nrVinculo         = (int)    $row['NRVINCULOM'];
            $date              = (string) $row['DATE'];
            $nrMarcacaoEntrada = (int)    $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int)    $row['NRMARCACAOSAIDA'];

            $status = $this->marcacaoService->fixaEntradaSaida(
                $nrOrg, $nrVinculo, $date,
                $nrMarcacaoEntrada, $nrMarcacaoSaida
            );

            if ($status['success']) {
                $dataset = [
                    'success' => true
                ];

                $response->addDataSet(new DataSet('data', $dataset));
            } else {
                $message = $status['message'];

                $response->addMessage(new Message($message));
            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getMarcacoesPontoDiaOpcoesFixarMarcacao(
        Request\Filter $request,
        Response       $response
    ) {
        try {
            $row = $request->getParameters();

            $nrOrg             = (int)    $row['NRORG'];
            $nrVinculo         = (int)    $row['NRVINCULOM'];
            $date              = (string) $row['DATE'];

            $marcacoes = $this->marcacaoService->getMarcacoesPontoDiaOpcoesFixarMarcacao(
                $nrOrg, $nrVinculo, $date
            );

            $response->addDataSet(new DataSet('data', $marcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function reverterEntradaSaida(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrOrg             = (int)    $row['NRORG'];
            $nrVinculo         = (int)    $row['NRVINCULOM'];
            $date              = (string) $row['DATE'];

            $success = $this->marcacaoService->reverterEntradaSaida(
                $nrOrg, $nrVinculo, $date
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('data', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDetalheAcompanhamento(
        Request\Filter $request,
        Response $response
    ) {
        try {
            $params = $request->getParameters();

            $nrOrg      = (int)   $params['NRORG'];
            $nrVinculo  = (int)   $params['NRVINCULOM'];
            $date       = (array) $params['DATAAPURACAO'];

            $detalhesAcompanhamento = $this->marcacaoService->getDetalheAcompanhamento(
                $nrOrg, $nrVinculo, $date
            );

            $response->addDataSet(new DataSet('data', $detalhesAcompanhamento));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}